package bls

import (
	"encoding/hex"
	"testing"

	"github.com/herumi/bls-eth-go-binary/bls"
)

func TestGetSafePublicKey(t *testing.T) {
	if bls.Init(bls.BLS12_381) != nil {
		t.Fatalf("Init")
	}

	var sec SecretKey
	pub, err := sec.GetSafePublicKey()
	if pub != nil || err == nil {
		t.Fatalf("sec must be zero")
	}
	sec.SetByCSPRNG()
	pub, err = sec.GetSafePublicKey()
	if pub == nil || err != nil {
		t.Fatalf("sec must be non-zero")
	}
}

func TestEthDraft07(t *testing.T) {
	secHex := "0000000000000000000000000000000000000000000000000000000000000001"
	msgHex := "61736466"
	sigHex := "b45a264e0d6f8614c4640ea97bae13effd3c74c4e200e3b1596d6830debc952602a7d210eca122dc4f596fa01d7f6299106933abd29477606f64588595e18349afe22ecf2aeeeb63753e88a42ef85b24140847e05620a28422f8c30f1d33b9aa"
	ethSignOneTest(t, secHex, msgHex, sigHex)
}

func ethSignOneTest(t *testing.T, secHex string, msgHex string, sigHex string) {
	var sec SecretKey
	if sec.DeserializeHexStr(secHex) != nil {
		t.Fatalf("bad sec")
	}
	var sig Sign
	if sig.DeserializeHexStr(sigHex) != nil {
		t.Logf("bad sig %v\n", sigHex)
		return
	}
	pub := sec.GetPublicKey()
	msg, _ := hex.DecodeString(msgHex)
	sig = *sec.SignByte(msg)
	if !sig.VerifyByte(pub, msg) {
		t.Fatalf("bad verify %v %v", secHex, msgHex)
	}
	s := sig.SerializeToHexStr()
	if s != sigHex {
		t.Fatalf("bad sign\nL=%v\nR=%v\nsec=%v\nmsg=%v", s, sigHex, secHex, msgHex)
	}
}

func TestBlsAggregateVerifyNoCheck(t *testing.T) {
	nTbl := []int{1, 2, 15, 16, 17, 50}
	for i := 0; i < len(nTbl); i++ {
		blsAggregateVerifyNoCheckTestOne(t, nTbl[i])
	}
}

func blsAggregateVerifyNoCheckTestOne(t *testing.T, n int) {
	t.Logf("blsAggregateVerifyNoCheckTestOne %v\n", n)
	pubs, sigs, msgs := makeMultiSig(n)
	if !bls.AreAllMsgDifferent(msgs) {
		t.Fatalf("bad msgs")
	}
	var aggSig Sign
	aggSig.Aggregate(sigs)
	if !aggSig.AggregateVerifyNoCheck(pubs, msgs) {
		t.Fatalf("bad AggregateVerifyNoCheck 1")
	}
	msgs[1] = 1
	if aggSig.AggregateVerifyNoCheck(pubs, msgs) {
		t.Fatalf("bad AggregateVerifyNoCheck 2")
	}
}

func makeMultiSig(n int) (pubs []PublicKey, sigs []Sign, msgs []byte) {
	msgSize := 32
	pubs = make([]PublicKey, n)
	sigs = make([]Sign, n)
	msgs = make([]byte, n*msgSize)
	for i := 0; i < n; i++ {
		var sec SecretKey
		sec.SetByCSPRNG()
		pubs[i] = *sec.GetPublicKey()
		msgs[msgSize*i] = byte(i)
		sigs[i] = *sec.SignByte(msgs[msgSize*i : msgSize*(i+1)])
	}
	return pubs, sigs, msgs
}
